package org.drools.bug;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.drools.event.rule.ObjectInsertedEvent;
import org.drools.event.rule.ObjectRetractedEvent;
import org.drools.event.rule.ObjectUpdatedEvent;
import org.drools.event.rule.WorkingMemoryEventListener;

@SuppressWarnings("unchecked")
public class MockEventListener implements WorkingMemoryEventListener {

	
	
	public static Inserted inserted(Object o) {
		return new Inserted(o);
	}

	public static Retracted retracted(Object o) {
		return new Retracted(o);
	}

	public static Updated updated(Object o) {
		return new Updated(o);
	}

	public static EventList eventList() {
		return new EventList();
	}

	public static class EventList {
		
		private ArrayList<Object> events = new ArrayList<Object>();
		
		public EventList() {
		}
		
		public EventList add(Object event) {
			events.add(event);
			return this;
		}
		
		public ArrayList<Object> events() {
			return events;
		}
		
	}
	
	private static boolean implementsEquals(Class<?> clazz) {
		boolean overridesEquals = false;
		while (clazz != null && !clazz.equals(Object.class)) {
			Method[] declaredMethods = clazz.getDeclaredMethods();
			for (Method m : declaredMethods) {
				Class<?>[] parameterTypes = m.getParameterTypes();
				if (m.getName().equals("equals") && parameterTypes != null 
						&& parameterTypes.length == 1 
						&& parameterTypes[0].equals(Object.class)
						&& m.getReturnType().equals(Boolean.TYPE)) {
					overridesEquals = true;
					break;
				}
			}
			clazz = clazz.getSuperclass();
		}
		return overridesEquals;
	}
	
	private static boolean implementsToString(Class<?> clazz) {
		boolean overridesEquals = false;
		while (clazz != null && !clazz.equals(Object.class)) {
			Method[] declaredMethods = clazz.getDeclaredMethods();
			for (Method m : declaredMethods) {
				Class<?>[] parameterTypes = m.getParameterTypes();
				if (m.getName().equals("toString") && parameterTypes != null 
						&& parameterTypes.length == 0 
						&& m.getReturnType().equals(String.class)) {
					overridesEquals = true;
					break;
				}
			}
			clazz = clazz.getSuperclass();
		}
		return overridesEquals;
	}
	
	
	public static class Inserted {

		private Object o;
		private int index=0;

		public Inserted(Object o) {
			this.o = o;
		}

		public Inserted(int index, Object o) {
			this.o = o;
			this.index = index;
		}

		public boolean equals(Object obj) {
			if (this == obj) {
				return true;
			}
			if (obj instanceof Inserted == false) {
				return false;
			}
			Inserted event = (Inserted) obj;
			if (index != event.index)
				return false;
			if (implementsEquals(o.getClass()))
				return o.equals(event.o);
			else
				return o.toString().equals(event.o.toString());
		}

		public String toString() {
			return new StringBuilder().append("Inserted(").append(o.toString())
					.append(')').toString();
		}

		public Object getObject() {
			return o;
		}
	}

	public static class Retracted {

		private Object o;
		private int index = 0;

		public Retracted(Object o) {
			this.o = o;
		}

		public Retracted(int index, Object o) {
			this.o = o;
			this.index = index;
		}

		public boolean equals(Object obj) {
			if (this == obj) {
				return true;
			}
			if (obj instanceof Retracted == false) {
				return false;
			}
			Retracted event = (Retracted) obj;
			if (index != event.index)
				return false;
			if (implementsEquals(o.getClass()))
				return o.equals(event.o);
			else
				return o.toString().equals(event.o.toString());
		}

		public String toString() {
			return new StringBuilder().append("Retracted(")
					.append(o.toString()).append(')').toString();
		}
		
		public Object getObject() {
			return o;
		}
	}

	public static class Updated {

		private Object o;
		private int index=0;

		public Updated(Object o) {
			this.o = o;
		}

		public Updated(int index, Object o) {
			this.o = o;
			this.index = index;
		}

		public boolean equals(Object obj) {
			if (this == obj) {
				return true;
			}
			if (obj instanceof Updated == false) {
				return false;
			}
			Updated event = (Updated) obj;
			if (index != event.index)
				return false;
			if (implementsEquals(o.getClass()))
				return o.equals(event.o);
			else
				return EqualsBuilder.reflectionEquals(o, event.o);
				//return o.toString().equals(event.o.toString());
		}

		public String toString() {
			return new StringBuilder().append("Updated(").append(o.toString())
					.append(')').toString();
		}
		public Object getObject() {
			return o;
		}
		
	}

	//private ArrayList<Object> events = new ArrayList<Object>();
	private List<Object> mock = mock(List.class);
	private int gotIndex = 0;
	private int expectedIndex = 0;
	private boolean recording = false;

	private Object copy(Object object) {
		if (object instanceof Cloneable) {
			try {
				Method method = object.getClass().getMethod("clone");
				return method.invoke(object);
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		} else if (implementsToString(object.getClass())) 
			return object.toString(); 
		else
			return object;
	}

	@Override
	public void objectInserted(ObjectInsertedEvent event) {
		Inserted inserted = new Inserted(gotIndex++, copy(event.getObject()));
		System.out.println(inserted);
		//events.add(inserted);
		if (recording)
			mock.add(inserted);
		else
			gotIndex--;
	}

	@Override
	public void objectRetracted(ObjectRetractedEvent event) {
		Retracted retracted = new Retracted(gotIndex++, copy(event.getOldObject()));
		System.out.println(retracted);
		//events.add(retracted);
		if (recording)
			mock.add(retracted);
		else
			gotIndex--;
	}

	@Override
	public void objectUpdated(ObjectUpdatedEvent event) {
		Updated updated = new Updated(gotIndex++, copy(event.getObject()));
		System.out.println(updated);
		//events.add(updated);
		if (recording)
			mock.add(updated);
		else
			gotIndex--;
	}

	public void expectedInsert(Object inserted2) {
		Inserted inserted = new Inserted(expectedIndex++, copy(inserted2));
//		System.out.println(inserted2);
		verify(mock).add(inserted);
		//assertEquals(inserted, mock.get(currentIndex++));
	}
	
	public void expectedUpdate(Object updated2) {
		Updated updated = new Updated(expectedIndex++, copy(updated2));
//		System.out.println(updated2);
		verify(mock).add(updated);
		//assertEquals(updated, mock.get(currentIndex++));
	}
	
	public void expectedRetract(Object retracted2) {
		Retracted retracted = new Retracted(expectedIndex++, copy(retracted2));
//		System.out.println(retracted2);
		verify(mock).add(retracted);
		//assertEquals(retracted, mock.get(currentIndex++));
	}
	
	
//	public ArrayList<Object> events() {
//		return events;
//	}
//
//	public void clear() {
//		events.clear();
//	}
//	
	public void end() {
		verifyNoMoreInteractions(mock);
	}
	
	public void startObserving() {
		recording = true;
	}

	public void stopObserving() {
		recording = false;
	}
	
}
