package org.drools.bug;

import java.io.File;

import org.drools.KnowledgeBase;
import org.drools.conf.AssertBehaviorOption;
import org.drools.conf.SequentialOption;
import org.drools.event.rule.WorkingMemoryEventListener;
import org.drools.runtime.StatefulKnowledgeSession;


public class RuleSetTester {

	private final SessionBuilder builder;
	private String testResourcesDir = "src/test/resources/";
	private final MockEventListener mockListener = new MockEventListener();
	private StatefulKnowledgeSession session = null;
	private long begin;
	
	public RuleSetTester() {
		builder = new SessionBuilder();
	}

	public RuleSetTester assertBehavior(AssertBehaviorOption assertBehavior) {
		builder.assertBehavior(assertBehavior);
		return this;
	}
	
	public RuleSetTester sequentialOption(SequentialOption sequentialOption) {
		builder.sequentialOption(sequentialOption);
		return this;
	}
	
	public RuleSetTester rulesEncoding(String encoding) {
		builder.rulesEncoding(encoding);
		return this;
	}
	
	public RuleSetTester rulesRootDir(String dir) {
		dir = dir.replace('\\', '/');
		if (dir.length() != 0 && dir.charAt(dir.length()-1) != '/')
			dir = dir + "/";
		this.testResourcesDir = dir;
		return this;
	}
	
	public RuleSetTester setKnowledgeBase(KnowledgeBase kb) {
		builder.setKnowledgeBase(kb);
		return this;
	}
	
	public RuleSetTester addTested(Class<?> clazz) {
		String name = clazz.getName().replace('.', '/');
		// try a drl
		String drl = testResourcesDir + name + ".drl";
		File drlf = new File(drl);
		if (drlf.exists())
			builder.appendDRL(drlf);
		else {
			String brl = testResourcesDir + name + ".brl";
			File brlf = new File(brl);
			if (brlf.exists())
				builder.appendDRL(drlf);
			else {
				String drt = testResourcesDir + name + ".drt";
				File drtf = new File(drt);
				if (drtf.exists())
					builder.appendDRT(drtf);
				else {
					throw new IllegalArgumentException("Cannot find the .brl, .drt or .drl corresponding to this class, looked for something like: " + drl);
				}
			}
		}
		return this;
		
	}
	
	public RuleSetTester addRuleFile(Class<?> clazz, String ruleFileName) {
		String name = clazz.getPackage().getName().replace('.', '/');
		String file = testResourcesDir + "/" + name + "/" + ruleFileName;
		return addRuleFile(file);
	}

	private RuleSetTester addRuleFile(String ruleFileName) {
		File file = new File(ruleFileName);
		if (!file.exists())
			throw new IllegalArgumentException("Cannot find file: " + ruleFileName);
		if (file.getName().toLowerCase().endsWith(".drl")) {
			builder.appendDRL(file);
//		} else if (file.getName().toLowerCase().endsWith(".brl")) {
//			builder.appendBRL(file);
		} else if (file.getName().toLowerCase().endsWith(".drt")) {
			builder.appendDRT(file);
		} else {
			throw new IllegalArgumentException("Unsupported rule file type: " + file.getPath());
		}
		return this;
	}
	
	public RuleSetTester setGlobal(String identifier, Object object) {
		builder.setGlobal(identifier, object);
		return this;
	}
	
	public RuleSetTester addWorkingMemoryEventListener(WorkingMemoryEventListener listener) {
		builder.addWorkingMemoryEventListener(listener);
		return this;
	}
	
	public RuleSetTester insert(Object object) {
		session.insert(object);
		return this;
	}
	
	public RuleSetTester startObserving() {
		mockListener.startObserving();
		return this;
	}
	
	public RuleSetTester stopObserving() {
		mockListener.stopObserving();
		return this;
	}
	
	public RuleSetTester expectedInsert(Object inserted) {
		mockListener.expectedInsert(inserted);
		return this;
	}
	
	public RuleSetTester expectedUpdate(Object updated) {
		mockListener.expectedUpdate(updated);
		return this;
	}
	
	public RuleSetTester expectedRetract(Object retracted) {
		mockListener.expectedRetract(retracted);
		return this;
	}
	
	public RuleSetTester startTest() {
		begin = System.currentTimeMillis();
		addWorkingMemoryEventListener(mockListener);
		session = builder.statefulSession();
		return this;
	}

	public RuleSetTester resetChronometer() {
		begin = System.currentTimeMillis();
		return this;
	}

	public RuleSetTester fireAllRules() {
		session.fireAllRules();
		return this;
	}

	public RuleSetTester endTest() {
		long end = System.currentTimeMillis();
		System.out.println("Elapsed (ms): " + (end-begin));
		session.dispose();
		mockListener.end();
		return this;
	}

}
