package org.drools.bug;

import java.io.File;
import java.io.FileInputStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.IOUtils;
import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.conf.AssertBehaviorOption;
import org.drools.conf.SequentialOption;
import org.drools.decisiontable.ExternalSpreadsheetCompiler;
import org.drools.decisiontable.InputType;
import org.drools.event.rule.WorkingMemoryEventListener;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;

public class SessionBuilder {

	private KnowledgeBuilder builder;
	private SequentialOption sequentialOption = SequentialOption.NO;
	private AssertBehaviorOption assertBehavior = AssertBehaviorOption.EQUALITY;
	private String encoding = "UTF-8";
	private HashMap<String, Object> globals = new HashMap<String, Object>();
	private ArrayList<WorkingMemoryEventListener> listeners = new ArrayList<WorkingMemoryEventListener>();

	private boolean verbose = true;
	private KnowledgeBase knowledgeBase;
	private ExternalSpreadsheetCompiler converter;
	
	public SessionBuilder() {
		try {
			builder = KnowledgeBuilderFactory.newKnowledgeBuilder();

		} catch (Exception e) {
			throw new RuntimeException(e);
		}

	}
	
	public SessionBuilder setGlobal(String identifier, Object object) {
		globals.put(identifier, object);
		return this;
	}
	
	public SessionBuilder addWorkingMemoryEventListener(WorkingMemoryEventListener listener) {
		listeners.add(listener);
		return this;
	}
	
	public SessionBuilder assertBehavior(AssertBehaviorOption assertBehavior) {
		this.assertBehavior = assertBehavior;
		return this;
	}
	
	public SessionBuilder sequentialOption(SequentialOption sequentialOption) {
		this.sequentialOption = sequentialOption;
		return this;
	}
	
	public SessionBuilder rulesEncoding(String encoding) {
		this.encoding = encoding;
		return this;
	}
	

//	public SessionBuilder appendBRL(File brl) {
//		try {
//			StringBuilder drl = new StringBuilder();
//			// append imports
//			drl.append(new String(IOUtils.toCharArray(FileUtils
//					.openInputStream(new File(brl.getParentFile(), "drools.package")), "UTF-8")));
//			// convert BRL to DRL
//			RuleModel model = BRXMLPersistence.getInstance().unmarshal(
//					new String(IOUtils.toCharArray(FileUtils
//							.openInputStream(brl), encoding)));
//			drl.append(BRDRLPersistence.getInstance().marshal(model));
//			if (verbose)
//				System.out.println(drl.toString());
//			builder.add(ResourceFactory.newReaderResource(new StringReader(drl
//					.toString())), ResourceType.DRL);
//			return this;
//		} catch (Exception e) {
//			throw new RuntimeException(e);
//		}
//	}

	public SessionBuilder appendDRL(File drlf) {
		try {
			String drl = new String(IOUtils.toCharArray(FileUtils
					.openInputStream(drlf), encoding));
			if (verbose)
				System.out.println(drl);
			builder.add(ResourceFactory.newReaderResource(new StringReader(drl)), ResourceType.DRL);
			return this;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public void appendDRT(File drtf) {
		try {
			// there must be a xls or csv next to it
			File csv = new File(FilenameUtils.removeExtension(drtf.getPath()) + ".csv");
			File xls = new File(FilenameUtils.removeExtension(drtf.getPath()) + ".xls");
			if (csv.exists() && xls.exists())
				throw new IllegalStateException(drtf.getPath() +": can't decide between csv and xls since both exist");
			if (!csv.exists() && !xls.exists())
				throw new IllegalStateException(drtf.getPath() +": no associated csv or xls");
			InputType type;
			File tableFile;
			if (csv.exists()) {
				tableFile = csv;
				type = InputType.CSV;
			} else {
				tableFile = xls;
				type = InputType.XLS;
			}
			if (converter == null)
				converter = new ExternalSpreadsheetCompiler();
	        String drl = converter.compile(new FileInputStream(tableFile), new FileInputStream(drtf), type , 2, 1);
			if (verbose)
				System.out.println(drl);
			builder.add(ResourceFactory.newReaderResource(new StringReader(drl)), ResourceType.DRL);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		
	}


	
//	public StatelessKnowledgeSession defaultSession() {
//		return session(true, assertBehavior, sequentialOption);
//	}
//
//	public StatelessKnowledgeSession session(boolean insertDefaultRules, KnowledgeBaseOption... options) {
//		if (knowledgeBase == null) {
//			if (builder.hasErrors()) {
//				throw new RuntimeException(builder.getErrors().toString());
//			}
//			KnowledgeBaseConfiguration configuration = KnowledgeBaseFactory
//					.newKnowledgeBaseConfiguration();
//			for (KnowledgeBaseOption option : options)
//				configuration.setOption(option);
//			knowledgeBase = KnowledgeBaseFactory
//					.newKnowledgeBase(configuration);
//			knowledgeBase.addKnowledgePackages(builder.getKnowledgePackages());
//		}
//		StatelessKnowledgeSession session = knowledgeBase.newStatelessKnowledgeSession();
//		for (Map.Entry<String, Object> entry : globals.entrySet()) {
//			session.setGlobal(entry.getKey(), entry.getValue());
//		}
//		for (WorkingMemoryEventListener listener : this.listeners)
//			session.execute(new AddEventListenerCommand(listener));
//		return session;
//	}

	public StatefulKnowledgeSession statefulSession() {
		if (knowledgeBase == null) {
			if (builder.hasErrors()) {
				throw new RuntimeException(builder.getErrors().toString());
			}
			KnowledgeBaseConfiguration configuration = KnowledgeBaseFactory
					.newKnowledgeBaseConfiguration();
			configuration.setOption(assertBehavior);
			configuration.setOption(sequentialOption);
			knowledgeBase = KnowledgeBaseFactory
					.newKnowledgeBase(configuration);
			knowledgeBase.addKnowledgePackages(builder.getKnowledgePackages());
		}
		StatefulKnowledgeSession session = knowledgeBase.newStatefulKnowledgeSession();
		for (Map.Entry<String, Object> entry : globals.entrySet()) {
			session.setGlobal(entry.getKey(), entry.getValue());
		}
		for (WorkingMemoryEventListener listener : this.listeners)
			session.addEventListener(listener);
		return session;
	}

	public void setKnowledgeBase(KnowledgeBase kb) {
		this.knowledgeBase = kb;
	}


}
